<?php

namespace XF301VB\Mvc;

use \XF\Http\Request;
use \XF\Mvc\RouteMatch;

class Router extends XFCP_Router
{
	// Sub-keys represent values of 'do'.
	// No key or 0 represents no 'do'.
	protected $simpleMap = [
		'login.php' => [
			'login',
			'lostpw' => 'lost-password'
		],
		'misc.php' => [
			'help',
			'bbcode' => 'help/bb-codes',
			'showrules' => 'help/terms'
		],
		'profile.php' => [
			'buddylist' => 'account/following',
			'editavatar' => 'account/account-details',
			'editoptions' => 'account/preferences',
			'editpassword' => 'account/security',
			'editprofile' => 'account/account-details',
			'editsignature' => 'account/signature',
			'ignorelist' => 'account/ignored',
			'privacy' => 'account/privacy'
		],
		'register.php' => [
			'register',
			'requestemail' => 'account-confirmation/resend'
		],
		'search.php' => [
			'search',
			'getnew' => 'whats-new/posts',
			'getdaily' => 'whats-new/posts',
		],
		'memberlist.php' => 'members',
		'showgroups.php' =>'members',
		'private.php' =>  'conversations',
		'subscription.php' => 'watched/threads',
		'usercp.php' => 'watched/threads',
		'activity.php' => 'whats-new/latest-activity',
		'faq.php' => 'help',
		'mobile.php' => 'index',
		'online.php' =>'online',
	];

	protected $actionMap = [
		'forumdisplay' => 'forum',
		'forums'       => 'forum',
		'f'            => 'forum',

		'showthread'   => 'thread',
		'printthread'  => 'thread',
		'threads'      => 'thread',
		't'            => 'thread',

		'showpost'     => 'post',
		'members'      => 'member',
		'private'      => 'privatemessage',
		'tags'         => 'tag',

		'blog'         => 'blogUser',
		'blogs'        => 'blogUser',
		'entry'        => 'blogEntry',
		'entries'      => 'blogEntry',
	];

	public function routeToController($path, Request $request = null)
	{
		// if $request is null, we're probably just testing a link
		if ($request === null)
		{
			return parent::routeToController($path, $request);
		}

		// Simple non-content-based routes like registration, memberlist...
		if (isset($this->simpleMap[$path]))
		{
			if (!is_array($this->simpleMap[$path]))
			{
				$this->simpleMap[$path] = [0 => $this->simpleMap[$path]];
			}

			if (is_array($this->simpleMap[$path]))
			{
				$do = $request->filter('do', 'str') ?: 0;

				if (isset($this->simpleMap[$path][$do]))
				{
					return $this->newRedirectRoute('simple-redirect', ['route_path' => $this->simpleMap[$path][$do]]);
				}
			}
		}

		$parts = explode('/', $path, 2);
		list($action) = $parts;

		switch ($action)
		{
			// vBulletin 3 and 4 non-rewrite URLs with INT IDs
			case 'showthread.php':
			case 'printthread.php':
			case 'showpost.php':
			case 'forumdisplay.php':
			case 'newthread.php':
			case 'member.php':
			case 'attachment.php':
			case 'private.php':
			case 'entry.php':
			case 'blog.php':
			{
				if ($match = $this->getNonRewriteMatch($action, $request))
				{
					return $match;
				}
				break;
			}

			// Announcement URLs
			case 'announcement.php':
			{
				$nodeId = $request->get('f');

				if ($nodeId)
				{
					return $this->newRedirectRoute('announcement', [
						'id_name' => 'f',
						'content_id' => $nodeId
					]);
				}
				else
				{
					return $this->newRedirectRoute('simple-redirect', [
						'route_path' => 'forums'
					]);
				}
			}

			// RSS feeds
			case 'external.php':
			{
				$nodeIds = $request->filter('forumids', 'str');
				$nodeId = (int) $nodeIds;

				if ($nodeId)
				{
					return $this->newRedirectRoute('forum-feed', [
						'content_id' => $nodeId,
					]);
				}
				else
				{
					return $this->newRedirectRoute('simple-redirect', [
						'route_path' => 'forums/index.rss',
					]);
				}
			}

			// vBulletin 3 and 4 non-rewrite URLs with STR IDs
			case 'tags.php':
			{
				if ($tag = $request->filter('tag', 'str'))
				{
					return $this->newRedirectRoute('tag', ['content_id' => $this->prepareStringForUrl($tag)]);
				}
				break;
			}

			// vBulletin 4 rewrite URLs
			case 'forums':
			case 'threads':
			case 'members':
			case 'entries':
			{
				if ($match = $this->getVbulletin4RewriteMatch($action, $request))
				{
					return $match;
				}
				break;
			}

			// vBulletin 5 rewrite URLs
			case 'forum':
			case 'member':
			case 'privatemessage':
			{
				if ($match = $this->getVbulletin5RewriteMatch($action, $request))
				{
					return $match;
				}
				break;
			}

			// Misc vBulletin 5 URLs
			case 'filedata':
			{
				if ($match = $this->getVbulletin5FileDataMatch($action, $request))
				{
					return $match;
				}
				break;
			}

			case 'search':
			{
				if ($match = $this->getVbulletin5SearchMatch($action, $request))
				{
					return $match;
				}
				break;
			}

			case 'blogs':
			{
				if ($match = $this->getVbulletinBlogMatch($action, $request))
				{
					return $match;
				}
				break;
			}

			// vB3 / vB4 archive URLs
			case 'archive':
			{
				if ($match = $this->getVbulletinArchiveMatch($action, $request))
				{
					return $match;
				}
				break;
			}
		}

		return parent::routeToController($path, $request);
	}

	protected function getNonRewriteMatch($action, Request $request)
	{
		$action = basename($action, '.php');
		$regexMatch = '#' . $action . '\.php(?:/|\?(?:do=\w+&)?)(?:(?P<id_name>[a-z]|[a-z]+id)=)?(?P<content_id>\d+)(?:-[a-z0-9-]+)?(?:(?:/|&)page=?(?P<page>\d+))?#i';

		if (preg_match($regexMatch, $request->getRequestUri(), $params))
		{
			if ($action == 'blog' && !empty($params['id_name']))
			{
				switch ($params['id_name'])
				{
					// TODO: 'blogUser' or 'blog-user'?
					case 'u': $action = 'blogUser'; break;
					case 'b': $action = 'blogEntry'; break;
				}
			}

			return $this->newRedirectRoute($action, $params);
		}
	}

	protected function getVbulletin4RewriteMatch($action, Request $request)
	{
		$regexMatch = '/\/' . $action . '(?:\/(?P<content_id>\d+)-[a-z0-9%!\-\$_\+\*\(\)\,]+(?:\/|\?|$))(?:page(?P<page>\d+))?/i';

		if (preg_match($regexMatch, $request->getRequestUri(), $params))
		{
			return $this->newRedirectRoute($action, $params);
		}
	}

	protected function getVbulletin5RewriteMatch($action, Request $request, $params = [])
	{
		$parts = array_reverse(preg_split('#/#', $request->getRequestUri(), -1, PREG_SPLIT_NO_EMPTY));

		// the URI will probably match something like /forum/something else and $action will be forum in that case,
		// so this will trigger a false positive
		$lastPart = end($parts);
		if ($lastPart && $lastPart === $action)
		{
			array_pop($parts);
		}

		foreach ($parts AS $i => $part)
		{
			if (preg_match('#^[%a-z0-9-]+\.\d+(?:\?|$)#i', $part) || $part === '-')
			{
				// this looks like an XF URL, so we will drop out and let the standard router take over
				return false;
			}
			else if (empty($params['page']) && preg_match('/^page(?P<page>\d+)/i', $part, $match))
			{
				// we have a page number, but we need more info, so continue the loop
				$params['page'] = $match['page'];
			}
			else if (preg_match('/^(?P<content_id>\d+)(?:-[%a-z0-9-]+)?(?:\?|$)/i', $part, $match))
			{
				// we have a content_id, so we don't need any more info for the route match
				$params['content_id'] = $match['content_id'];
				switch ($action)
				{
					// in vB5, forum/a/b/c/d/(int)-name corresponds to a thread, so point the action there
					// and as we import blog entries as threads, point those too...
					case 'forum':
					case 'blogs':
						$action = 'thread';
						break;
				}
				break;
			}
			else if (preg_match('#^(?P<node_name>[a-z0-9-]+)$#i', $part, $match) && empty($params['node_name']))
			{
				// we've got a node name, so we don't need any more info for the route match
				$params['node_name'] = $match['node_name'];
			}
		}

		if (!empty($params['content_id']) || !empty($params['node_name']))
		{
			return $this->newRedirectRoute($action, $params);
		}
	}

	protected function getVbulletin5FileDataMatch($action, Request $request)
	{
		$input = $request->filter([
			'id' => 'int',
			'filedataid' => 'int',
			'photoid' => 'int'
		]);

		return $this->newRedirectRoute('attachment',
		[
			'content_id' => $input['id'] ?: $input['photoid'],
			'filedata_id' => $input['filedataid']
		]);
	}

	protected function getVbulletin5SearchMatch($action, Request $request)
	{
		// only treat this as an import redirect if we have 'searchJSON' specified
		if ($searchJson = $request->filter('searchJSON', 'json-array'))
		{
			if (array_key_exists('tag', $searchJson))
			{
				return $this->newRedirectRoute('tag',
					['content_id' => $this->prepareStringForUrl($searchJson['tag'][0])]);
			}
			else if (array_key_exists('prefix', $searchJson))
			{
				return $this->newRedirectRoute('prefix',
					['content_id' => $this->prepareStringForUrl($searchJson['prefix'][0])]);
			}
		}
	}

	protected function getVbulletinBlogMatch($action, Request $request)
	{
		// This has to check for a valid vB4 route (blogs/id-name)
		// before falling back to a vB5 route (blogs/a/b/c/id-name)

		if ($match = $this->getVbulletin4RewriteMatch($action, $request))
		{
			return $match;
		}

		if ($match = $this->getVbulletin5RewriteMatch($action, $request))
		{
			return $match;
		}
	}

	protected function getVbulletinArchiveMatch($action, Request $request)
	{
		if (preg_match('#/archive/index\.php(/|\?)(?P<action>f|t)-(?P<content_id>\d+)(?:-p-(?P<page>\d+))?\.html#i', $request->getRequestUri(), $params))
		{
			return $this->newRedirectRoute($params['action'], $params);
		}
	}

	protected function newRedirectRoute($action, array $params)
	{
		$filteredParams = [];
		foreach ($params AS $key => $value)
		{
			if (is_string($key))
			{
				$filteredParams[$key] = $value;
			}
		}

		$match = new RouteMatch();
		$match->setController('XF301VB:Redirect');
		$match->setAction($this->getMappedAction($action));
		$match->setParams($filteredParams);

		return $match;
	}

	protected function getMappedAction($action)
	{
		if (isset($this->actionMap[$action]))
		{
			return $this->actionMap[$action];
		}

		return $action;
	}
}