<?php

namespace XF301VB\Option;

use XF\Option\AbstractOption;

class ImportVbTable extends AbstractOption
{
	public static function renderOption(\XF\Entity\Option $option, array $htmlParams)
	{
		$choices = ['' => \XF::phrase('none')] + self::getTableChoices();

		return self::getSelectRow($option, $htmlParams, $choices, $option->option_value);
	}

	public static function verifyOption(&$value, \XF\Entity\Option $option)
	{
		if ($value === '')
		{
			return true;
		}

		if (!in_array($value, self::getTableChoices()))
		{
			$option->error(\XF::phrase('specified_table_does_not_exist'), $option->option_id);
			return false;
		}

		$columns = \XF::app()->db()->fetchAllColumn('SHOW COLUMNS FROM ' . $value);
		foreach (['content_type', 'old_id', 'new_id'] AS $column)
		{
			if (!in_array($column, $columns))
			{
				$option->error(\XF::phrase('specified_table_does_not_appear_to_be_import_log'), $option->option_id);
				return false;
			}
		}

		return true;
	}

	protected static function getTableChoices()
	{
		// Hide tables from our own add-ons
		// But XFRM's tables aren't public, so just hide any xf_rm_* tables

		$xf = new \XF\Install\Data\MySql();
		$schemaTables = $xf->getTables();

		$choices = [];

		foreach (\XF::app()->db()->fetchAllColumn('SHOW TABLES') AS  $table)
		{
			if (!isset($schemaTables[$table]) && substr($table, 0, 6) != 'xf_rm_')
			{
				$choices[$table] = $table;
			}
		}

		return $choices;
	}
}