<?php

namespace XF301VB\Pub\Controller;

use XF\Mvc\ParameterBag;
use XF\Mvc\Controller;

class Redirect extends Controller
{
	// These actions are fired from XF301VB\Mvc\Router

	public function actionForum(ParameterBag $params)
	{
		$oldId = $params->node_name ?: $params->content_id;
		if (strlen(strval($oldId)) > 50)
		{
			$oldId = md5($oldId);
		}

		return $this->redirectContent($params, 'node', 'XF:Node', 'forums', $oldId);
	}

	public function actionAnnouncement(ParameterBag $params)
	{
		return $this->redirectContent($params, 'node', 'XF:Node', 'forums');
	}

	public function actionForumFeed(ParameterBag $params)
	{
		return $this->redirectContent($params, 'node', 'XF:Node', 'forums/index.rss');
	}

	public function actionNewthread(ParameterBag $params)
	{
		return $this->redirectContent($params, 'node', 'XF:Node', 'forums/post-thread');
	}

	public function actionThread(ParameterBag $params)
	{
		if ($postId = $this->filter('p', 'int'))
		{
			return $this->rerouteController(__CLASS__, 'post', ['content_id' => $postId]);
		}

		return $this->redirectContent($params, 'thread', 'XF:Thread', 'threads');
	}

	public function actionPost(ParameterBag $params)
	{
		return $this->redirectContent($params, 'post', 'XF:Post', 'posts');
	}

	public function actionMember(ParameterBag $params)
	{
		return $this->redirectContent($params, 'user', 'XF:User', 'members');
	}

	public function actionPrivateMessage(ParameterBag $params)
	{
		return $this->redirectContent($params, 'conversation', 'XF:ConversationMaster', 'conversations');
	}

	public function actionAttachment(ParameterBag $params)
	{
		if ($params->filedataid)
		{
			return $this->redirectContent($params, 'attachment_filedata', 'XF:Attachment', 'attachments',
				$params->filedata_id);
		}
		else
		{
			return $this->redirectContent($params, 'attachment', 'XF:Attachment', 'attachments');
		}
	}

	public function actionTag(ParameterBag $params)
	{
		// we don't need a lookup for a tag
		return $this->redirectPermanently(
			$this->buildLink('tags', ['tag_url' => $params->content_id])
		);
	}

	public function actionBlogUser(ParameterBag $params)
	{
		return $this->redirectContent($params, 'blog_user', 'XF:Node', 'forums');
	}

	public function actionBlogEntry(ParameterBag $params)
	{
		return $this->redirectContent($params, 'blog', 'XF:Thread', 'threads');
	}

	public function actionPrefix(ParameterBag $params)
	{
		// TODO: this doesn't really correspond to a standard XF action, actually
		return $this->error('This page does not exist');
	}

	public function actionSimpleRedirect(ParameterBag $params)
	{
		return $this->redirectPermanently($this->buildLink($params->route_path));
	}

	protected function redirectContent(ParameterBag $params, $type, $finder, $route, $oldId = null)
	{
		if ($oldId === null)
		{
			$oldId = $params->content_id;
		}

		try
		{
			$newId = $this->getLookupRepo()->lookupId($type, $oldId);
		}
		catch (\Exception $e)
		{
			$this->app()->logException($e);
			return $this->error($e->getMessage());
		}

		if ($newId)
		{
			if ($content = $this->finder($finder)->whereId($newId)->fetchOne())
			{
				$method = $content instanceof \XF\Entity\User ? 'canViewBasicProfile' : 'canView';
				if (is_callable([$content, $method]) && !$content->$method())
				{
					return $this->noPermission();
				}

				if ($content instanceof \XF\Entity\Node && $route === 'forums')
				{
					$route = $content->getRoute();
				}

				$extraParams = [];

				if ($params instanceof ParameterBag)
				{
					if ($params->page)
					{
						$extraParams['page'] = $params->page;
					}
				}

				return $this->redirectPermanently($this->buildLink('canonical:' . $route, $content, $extraParams));
			}
		}

		return $this->error(\XF::phrase('requested_page_not_found'), 404);
	}

	/**
	 * @return \XF301VB\Repository\ImportLookup
	 */
	protected function getLookupRepo()
	{
		return $this->repository('XF301VB:ImportLookup');
	}
}